from typing import List, Optional, Dict

QQ_GROUP_ID_LIST: Optional[Dict[str, List[str]]] = {}
QQ_GUILD_ID_LIST: Optional[Dict[str, List[str]]] = {}

ONEBOT_GROUP_ID_LIST: Optional[Dict[str, List[str]]] = {}
ONEBOT_GUILD_ID_LIST: Optional[Dict[str, List[str]]] = {}
